// *** Lucifer Windows CE ***
// Version 1.2
// Franois PESSAUX June 2001
// This program is freeware.
// Use its sources as you whant provided that you leave this copyright unchanged
// and that you drop a small word telling that I'm the author of these sources.
// This program is heavily based on
// David A. Barrett's implementation od Cipher 3.0.
// (barrett@boulder.Colorado.EDU). Thank's to him for having provided his code !
// Thank's a lot.
//   --Francois

#include "stdafx.h"
#include "resource.h"



const TBBUTTON cbar_buttons_def[] = {
//  BitmapIndex       Command   State            Style       UserData String
	{0, IDC_BUTTON_ENCRYPT, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0},
  {1, IDC_BUTTON_DECRYPT, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0}
} ;

const LPTSTR cbar_tooltips_def[] = {
	TEXT("Encrypt a file"),
	TEXT("Decrypt a file")
} ;

#define	IO_BUFSIZE	1024		/* small for data cache speed */
#define KEYBUFLEN	129		/* allow long keys */


typedef struct {
  char	   iv[DES_BLOCKSIZE];
  keyType key;
  int	   ivlen;
  int	   decr;
} KeyItem;

typedef unsigned char block[DES_BLOCKSIZE];
/*
 * Keys which could accidently reproduce the plaintext with multiple-encryption
 */
block badKeys[16] = {
  { 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0, 0x00 },
  { 0x00, 0xe0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf0 }, 
  { 0xfe, 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e },
  { 0x1e, 0xfe, 0x1e, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe }, 
  { 0xe0, 0x1e, 0xe0, 0x1e, 0xf0, 0x0e, 0xf0, 0x0e },
  { 0x1e, 0xe0, 0x1e, 0xe0, 0x0e, 0xf0, 0x0e, 0xf0 }, 
  { 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe },
  { 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0x00 },
  { 0x00, 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e },
  { 0x1e, 0x00, 0x1e, 0x00, 0x0e, 0x00, 0x0e, 0x00 }, 
  { 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0, 0xfe },
  { 0xfe, 0xe0, 0xfe, 0xe0, 0xfe, 0xf0, 0xfe, 0xf0 }, 
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },	/* self-dual */
  { 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe },	/* self-dual */
  { 0xe0, 0xe0, 0xe0, 0xe0, 0xf0, 0xf0, 0xf0, 0xf0 },	/* self-dual */
  { 0x1e, 0x1e, 0x1e, 0x1e, 0x0e, 0x0e, 0x0e, 0x0e },	/* self-dual */
};




/*
 * Return nonzero if the key is one of the well-known "weak" keys (FIPS Pub 74)
 */
int weakKey (char *keybits)
{
	block *keyp = &(badKeys[0]) ;	/* common compiler bug here */
  unsigned count = sizeof(badKeys) / sizeof(badKeys[0]) ;
  unsigned i ;

  do {
    for (i = 0; i < DES_BLOCKSIZE; i++) {
      if ((((*keyp)[i] ^ keybits[i]) & 0xfe) != 0) goto notfound ;
    }
    break ;
		notfound:
		keyp++ ;
  } while (--count != 0) ;

  return (count) ;
}



/*
 * Because we are using CFB, we always encrypt.  Since we'd like to be
 * able to run the chips in either mode, perhaps CBC would be better.
 * If CBC is used though, the last block must be handled in OFB mode.  Sigh.
 *
 * Returns:  
 *  -1 - out of memory (program exits, and error reported)
 *   0 - success
 *   1 - key is a weak key
 */
int setKey (keyType *kp, char *keybits, char *keystr, int mode)
{
	int res ;

  if (keystr != (char *) 0) {
    desKey (keybits, keystr, ' ') ;		/* space padded */
    memset (keystr, '\0', strlen (keystr)) ;	/* blank out key asap */
  }

  res = weakKey (keybits) ;

  makeKey (kp, keybits, DES_BLOCKSIZE, mode) ;	/* 0 means encrypt mode */
  memset (keybits, '\0', DES_BLOCKSIZE) ;  	/* zero out bits asap */

  if (*kp == (keyType) 0) {
    // Couldn't allocate memory for encryption key
    return (-1) ;
  }

  return (res) ;
}



void EDCryption (HWND w_handle, BOOL encryption_flag, LPTSTR in_file_name, char *keybuf)
{
	KeyItem key_item ;
	char bufs[2][IO_BUFSIZE] ;
	char *chp, *ibuf = bufs[1], *obuf = bufs[0];
  char keybits[DES_BLOCKSIZE] ;
	TCHAR out_file_name[MAX_STRINGLEN] ;

	int i ;
	DWORD rcount, wcount ;
	HANDLE in_file = INVALID_HANDLE_VALUE, out_file = INVALID_HANDLE_VALUE ;

	in_file = CreateFile (in_file_name, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL) ;
	if (in_file == INVALID_HANDLE_VALUE) {
		MessageBox (w_handle, TEXT ("Unable to open input file"), TEXT ("Error"),
			MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;
		return ;
		}

	// Generate the file name according if we are in an encryption or decryption mode.
	if (encryption_flag) {
		// Decryption. So extract the output file name from the archive
		// First the original file name length EXCLUDING the final `\0`
		ReadFile (in_file, &i, sizeof (int), &rcount, NULL) ;
		ReadFile (in_file, out_file_name, i * sizeof (TCHAR), &rcount, NULL) ;
		out_file_name[i] = 0 ;
		}
	else {
		// Encryption. So file name is the current filename + ".luc" extension.
		wcscpy (out_file_name, in_file_name) ;
		wcscat (out_file_name, TEXT (".luc")) ;
		}

	// Test if the file already exists...
	out_file = CreateFile (out_file_name, GENERIC_WRITE, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL) ;
	if (out_file != INVALID_HANDLE_VALUE) {
		// The file already exists. User must confirm its destruction.
		OPENFILENAME of ;

		CloseHandle (out_file) ;
		MessageBox (w_handle, TEXT ("A file with this name already exists. Chose another name or confirm its destruction."), TEXT ("Error"),
			MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;

		memset (&of, 0, sizeof (of)) ;
		of.lStructSize = sizeof (of) ;
		of.hwndOwner = NULL ;
		of.lpstrFile = out_file_name ;
		of.nMaxFile = MAX_STRINGLEN ;
		of.lpstrFilter = TEXT ("All Files (*.*)\0*.*\0\0") ;
		of.Flags = 0 ;
		if (GetSaveFileName (&of) == 0) {
			MessageBox (w_handle, TEXT ("Operation aborted"), TEXT ("Info"),
				MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONINFORMATION) ;
			CloseHandle (in_file) ;
			return ;
			}
		}

	// Now finally create the output file.
	out_file = CreateFile (out_file_name, GENERIC_WRITE, FILE_SHARE_READ, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL) ;
	if (out_file == INVALID_HANDLE_VALUE) {
			MessageBox (w_handle, TEXT ("Unable to open output file."), TEXT ("Error"),
				MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;
			CloseHandle (in_file) ;
			return ;
		}

	// And finally perform the great job...
	if (!encryption_flag) {
		// Encryption. So first store the length of the original
		// filename EXCLUDING the final `\0`
		i = wcslen (in_file_name) ;
		WriteFile (out_file, &i, sizeof (int), &wcount, NULL) ;
		WriteFile (out_file, in_file_name, i * sizeof (TCHAR), &wcount, NULL) ;
		}

	// Generate the encryption/decryption key
	key_item.decr = encryption_flag ;			// FALSE = encrypt, TRUE = decrypt
	key_item.ivlen = DES_BLOCKSIZE ;
	key_item.key = NULL ;									// Must be NULL !!!
  memset (key_item.iv, '\0', DES_BLOCKSIZE) ;
	i = setKey (&key_item.key, keybits, keybuf, 0) ;

	// We disallow weak keys only during encryption in case where the
	// file was encrypted by another compatible program that allowed
	// encryption with a weak key.
	if ((i) && (!encryption_flag)) {
		MessageBox (w_handle, TEXT ("Weak key used. Encryption rejected."),
			TEXT ("Error"),	MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;
		desDestroyKey (&key_item.key) ;
		CloseHandle (in_file) ;
		CloseHandle (out_file) ;
		return ;
		}

	do {
    ReadFile (in_file, obuf, IO_BUFSIZE, &rcount, NULL) ;
    if (rcount <= 0) break ;
    chp = obuf ;
    obuf = ibuf ;
    ibuf = chp ;
    key_item.ivlen = desCFB (obuf, ibuf, rcount, key_item.iv, key_item.ivlen,
			key_item.key, key_item.decr) ;
		WriteFile (out_file, obuf, rcount, &wcount, NULL) ;
  } while (wcount == rcount) ;

	// Perform the cleanup ASAP.
	desDestroyKey (&key_item.key) ;
	memset (bufs[0], 0, IO_BUFSIZE) ;
	memset (bufs[1], 0, IO_BUFSIZE) ;
	CloseHandle (in_file) ;
	CloseHandle (out_file) ;

	// Now display a bit of information.
	if (encryption_flag) {
	  if (MessageBox (w_handle, TEXT ("Launch default viewer ?"), TEXT ("Info"),
			MB_APPLMODAL|MB_SETFOREGROUND|MB_YESNO|MB_ICONINFORMATION|MB_ICONQUESTION) == IDYES) {
			SHELLEXECUTEINFO shell_info ;
			 
			 // Launch the default viewer.
			 shell_info.cbSize = sizeof (shell_info) ;
			 shell_info.fMask = 0 ;
			 shell_info.hwnd = NULL ;
			 shell_info.lpVerb = NULL ;								// Defaults to verb Open.
			 shell_info.lpFile = out_file_name ;
			 shell_info.lpParameters = NULL ;
			 shell_info.lpDirectory = NULL ;
			 shell_info.nShow = SW_SHOWNORMAL ;
			 shell_info.hInstApp = app_instance_handle ;
			 if (! ShellExecuteEx (&shell_info)) {
				 MessageBox (w_handle, TEXT ("An error occured while launching default viewer"),
					 TEXT ("Error"), MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;
				 }																			// End of if (! ShellExecuteEx (...
			}																					// End of if (MessageBox (w_handle, ...
		}																						// End of if (encryption_flag)
	else {
		// Remove the original source file if the user wants so.
		if (MessageBox (w_handle, TEXT ("Encryption complete. Remove original file ?"),
			TEXT ("Info"), MB_APPLMODAL|MB_SETFOREGROUND|MB_YESNO|MB_ICONINFORMATION|MB_ICONQUESTION)	== IDYES)
			DeleteFile (in_file_name) ;
		}																						// End of the else of if (encryption_flag)
}




LRESULT CALLBACK PasswdDialogHandler (HWND w_handle, UINT message,
																			WPARAM w_param, LPARAM l_param)
{
	int wm_id, wm_event ;

  wm_id    = LOWORD (w_param) ;
  wm_event = HIWORD (w_param) ;

  switch (message) {
	case WM_INITDIALOG:
		SetWindowText (GetDlgItem (w_handle, IDC_STATIC_PASSWD), tmp_unicode_string) ;
		SetFocus (GetDlgItem (w_handle, IDC_EDIT_PASSWD)) ;
		break ;												// End of WM_INITDIALOG

	case WM_COMMAND:
		switch (wm_id) {
		case IDC_BUTTON_PASSWD_OK:
			// Close the modal dialog
			GetWindowText (GetDlgItem (w_handle, IDC_EDIT_PASSWD),
				tmp_unicode_string, KEYBUFLEN-1) ;
			wcstombs ((char*) tmp_char_string, tmp_unicode_string, KEYBUFLEN-1) ;
			memset (tmp_unicode_string, 0, KEYBUFLEN * sizeof (TCHAR)) ;
			EndDialog (w_handle, TRUE) ;
			return (TRUE) ;							// Tell that we have fully handled the message
			break ;											// End of IDC_BUTTON_PASSWD_OK

		case IDC_BUTTON_PASSWD_CANCEL:
			// Close the modal dialog
			EndDialog (w_handle, TRUE) ;
			tmp_char_string[0] = '\0' ;
			memset (tmp_unicode_string, 0, KEYBUFLEN * sizeof (TCHAR)) ;
			return (TRUE) ;							// Tell that we have fully handled the message
			break ;											// End of IDC_BUTTON_PASSWD_CANCEL
		}															// End of switch (wm_id)
		break ;												// End of WM_COMMAND
	}																// End of switch (message)

  // Tells that we did not handle the message
	// Force default handling by the system (because we're
	// in the case of a dialog handler). Calling DefWindowProc is only the
	// protocol for window handlers
  return (FALSE) ;
}



void CheckAuthentification (HWND w_handle, BOOL operation, TCHAR *file_name)
{
	char passwd1[KEYBUFLEN] ;
	char passwd2[KEYBUFLEN] ;

	// Use the undocummented flag SIPF_DISABLECOMPLETION to avoid passords to
	// be stored in the word completion dictionary !!!
	SipShowIM (SIPF_ON|SIPF_DISABLECOMPLETION) ;
	// Open a modal dialog
	LoadString (app_instance_handle, IDS_PASSWD_1, tmp_unicode_string, MAX_STRINGLEN) ;
	DialogBox (app_instance_handle, MAKEINTRESOURCE (IDD_DIALOG_PASSWD),
				w_handle, (DLGPROC) PasswdDialogHandler) ;
	// The current passord is stored in clear in 'tmp_char_string'.
	strcpy (passwd1, tmp_char_string) ;
	if (tmp_char_string[0] == '\0') {
		MessageBox (w_handle, TEXT ("Operation aborted."), TEXT ("Info"),
			MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONINFORMATION) ;
		goto cleanup ;
		}
	
	if (!operation) {											// FALSE means encryption
		// Re-Open a modal dialog for confirmation
		LoadString (app_instance_handle, IDS_PASSWD_2, tmp_unicode_string, MAX_STRINGLEN) ;
		DialogBox (app_instance_handle, MAKEINTRESOURCE (IDD_DIALOG_PASSWD),
		w_handle, (DLGPROC) PasswdDialogHandler) ;
		// The current passord is stored in clear in 'tmp_char_string'.
		strcpy (passwd2, tmp_char_string) ;
		// Now test that the 2 passwords match.
		if (strcmp (passwd1, passwd2) != 0) {
			MessageBox (w_handle, TEXT ("Passwords do not match. Operation aborted."), TEXT ("Error"),
				MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONERROR) ;
			goto cleanup ;
		}
	}

	EDCryption (w_handle, operation, file_name, passwd1) ;
	
  cleanup:
	// Clear the ASCII passwords
	memset (tmp_char_string, 0, MAX_STRINGLEN * sizeof (char)) ;
	memset (passwd1, 0, KEYBUFLEN * sizeof (char)) ;
	memset (passwd2, 0, KEYBUFLEN * sizeof (char)) ;
	SipShowIM (SIPF_OFF) ;
}



LRESULT CALLBACK MainWindowHandler (HWND w_handle, UINT message, WPARAM w_param,
																		LPARAM l_param)
{
  int wm_id, wm_event ;

  wm_id    = LOWORD (w_param) ;
  wm_event = HIWORD (w_param) ;

	  switch (message) {
		case WM_CREATE: {
			HWND cbar_handle ;

			// Add the command bar.
			cbar_handle = CommandBar_Create (app_instance_handle, w_handle, 1000) ;
			CommandBar_AddBitmap (cbar_handle, app_instance_handle, IDB_BITMAP_CBAR, 2, 0, 0) ;
			CommandBar_AddButtons (cbar_handle, 2, cbar_buttons_def) ;
			CommandBar_AddToolTips (cbar_handle, 2, cbar_tooltips_def) ;
			CommandBar_AddAdornments (cbar_handle, CMDBAR_HELP, NULL) ;
			}													// End of local block for WM_CREATE
			break ;										// End of WM_CREATE

		case WM_COMMAND:
			switch (wm_id) {
			case IDC_BUTTON_DECRYPT:
			case IDC_BUTTON_ENCRYPT:
				if (l_param == NULL) {
					TCHAR file_name[MAX_STRINGLEN] = TEXT ("") ;
					wcscpy (file_name, GetCommandLine ()) ;
					CheckAuthentification (w_handle, TRUE, file_name) ;
				}
				else {
					TCHAR file_name[MAX_STRINGLEN] = TEXT ("") ;
					OPENFILENAME of ;
					const LPTSTR encrypt_filter = TEXT ("All Documents (*.*)\0*.*\0\0") ;
					const LPTSTR decrypt_filter = TEXT ("Lucifer Files (*.luc)\0*.luc\0\0") ;

					memset (&of, 0, sizeof (of)) ;
					of.lStructSize = sizeof (of) ;
					of.hwndOwner = NULL ;
					of.lpstrFile = file_name ;
					of.nMaxFile = MAX_STRINGLEN ;
					if (wm_id == IDC_BUTTON_DECRYPT) of.lpstrFilter = decrypt_filter ;
					else of.lpstrFilter = encrypt_filter ;
					of.Flags = OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST ;
					if (! GetOpenFileName (&of)) {
						MessageBox (w_handle, TEXT ("Operation aborted"), TEXT ("Info"),
							MB_APPLMODAL|MB_SETFOREGROUND|MB_OK|MB_ICONINFORMATION) ;
						return (TRUE) ;
					}

					if (wm_id == IDC_BUTTON_DECRYPT)
						CheckAuthentification (w_handle, TRUE, file_name) ;
					else CheckAuthentification (w_handle, FALSE, file_name) ;
					return (TRUE) ;				// Tell that we have fully handled the message
					}											// End of else of if (l_param == NULL)
				break ;									// End of IDC_BUTTON_ENCRYPT
			}													// End of switch (wm_id)
			break ;										// End of WM_COMMAND

			case WM_HELP:
				CreateProcess (TEXT("peghelp"), TEXT("Lucifer.htm"), NULL, NULL,
					FALSE, 0, NULL, NULL, NULL, NULL) ;
				return (TRUE) ;
				break ;									// End of WM_HELP

			case WM_CLOSE:
				// Just triggers a WM_QUIT message that will cause exiting the mainloop.
				PostQuitMessage (0) ;
				return (TRUE) ;
				break ;									// End of WM_CLOSE
		}														// End for switch (message)

		// Remember, window handler must call DefWindowProc to handle
		// the remaining messages. Returning TRUE or FALSE is only the protocol
		// for dialog handlers !
		return (DefWindowProc (w_handle, message, w_param, l_param)) ;
}




int WINAPI WinMain (HINSTANCE i_handle, HINSTANCE prev_i_handle,
		    LPTSTR cmd_line, int w_aspect)
{
	MSG msg ;
	WNDCLASS wc ;
	HWND w_handle ;
	SIPINFO si ;
	int iDelta ;

	// Initialize the global variable containing the class name of the window
	LoadString (i_handle, IDS_W_CLASS_NAME, w_class_name, MAX_STRINGLEN) ;
	app_instance_handle = i_handle ;

	// Check if the program is already running. If yes then focus on the window
	w_handle = FindWindow (w_class_name, NULL) ;
	if (w_handle) {
		// Assumes decryption if there is a filename on the command line
		if (wcscmp (cmd_line, TEXT ("")) != 0)
			SendMessage (w_handle, WM_COMMAND, MAKEWPARAM(IDC_BUTTON_DECRYPT, 1), NULL) ;
		ShowWindow (w_handle, w_aspect) ;
		SetForegroundWindow (w_handle) ;
		return (0) ;
	}

	// Register the window.
	if (! prev_i_handle) {
		wc.style = CS_HREDRAW|CS_VREDRAW ;
		wc.lpfnWndProc = (WNDPROC) MainWindowHandler ;
		wc.cbClsExtra = 0 ;
    wc.cbWndExtra = 0 ;
    wc.hIcon = (HICON) LoadIcon (app_instance_handle, MAKEINTRESOURCE (IDI_ICON_LUCIFER)) ;
    wc.hInstance = i_handle ;
    wc.hCursor = NULL ;
    wc.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
    wc.lpszMenuName = NULL ;
    wc.lpszClassName = w_class_name ;
    RegisterClass (&wc) ;
  }

	// Loads the DLL associated with common controls.
	InitCommonControls () ;

  // Create the window. Consider the menu at the bottom and the SIP
	si.cbSize = sizeof (si) ;
	SHSipInfo (SPI_GETSIPINFO, 0, &si, 0) ;

	iDelta = (si.fdwFlags & SIPF_ON) ? 0 : MENU_HEIGHT ;
  w_handle = CreateWindow (w_class_name, TEXT ("Lucifer 1.2"),
			   WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT,
			   si.rcVisibleDesktop.right - si.rcVisibleDesktop.left,
				 si.rcVisibleDesktop.bottom - si.rcVisibleDesktop.top - iDelta,
				 NULL, NULL, i_handle, NULL) ;

	SendMessage (w_handle, WM_SETICON, ICON_SMALL,
		(LPARAM) LoadImage (app_instance_handle,
		MAKEINTRESOURCE(IDI_ICON_LUCIFER), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR)) ;

	// Finally display the main window
  ShowWindow (w_handle, w_aspect) ;
  UpdateWindow (w_handle) ;

	// Assumes decryption if there is a filename on the command line
	if (wcscmp (cmd_line, TEXT ("")) != 0)
		SendMessage (w_handle, WM_COMMAND, MAKEWPARAM(IDC_BUTTON_DECRYPT, 1), NULL) ;

  // Entering main message loop
  while (GetMessage (&msg, NULL, 0, 0)) {
    TranslateMessage (&msg) ;
    DispatchMessage (&msg) ;
  }
  
	// Clear password buffers to be secure
	memset (tmp_unicode_string, 0, MAX_STRINGLEN * sizeof (TCHAR)) ;
	memset (tmp_char_string, 0, MAX_STRINGLEN * sizeof (char)) ;
  return (msg.wParam) ;
}
